// review.model.ts
import {
  DataTypes,
  Model,
  InferAttributes,
  InferCreationAttributes,
  CreationOptional,
  NonAttribute,
} from "@sequelize/core";
import {
  Table,
  Attribute,
  PrimaryKey,
  AutoIncrement,
  BelongsTo,
  CreatedAt,
  UpdatedAt,
  HasMany,
  Default,
} from "@sequelize/core/decorators-legacy";
import Product from "./product.model"; // Assuming reviews are associated with products
import User from "./user.model";

@Table({ tableName: "Reviews" })
export default class Review extends Model<
  InferAttributes<Review>,
  InferCreationAttributes<Review>
> {
  @PrimaryKey
  @AutoIncrement
  @Attribute(DataTypes.INTEGER)
  declare id: CreationOptional<number>;

  @Attribute(DataTypes.INTEGER)
  declare productId: number;

  @BelongsTo(() => Product, "productId")
  declare product: NonAttribute<Product>;

  @Attribute(DataTypes.INTEGER)
  declare userId: number;

  @BelongsTo(() => User, "userId")
  declare user: NonAttribute<User>;

  @Attribute(DataTypes.TEXT)
  declare text: string;

  @Attribute(DataTypes.DECIMAL(10, 1))
  declare rating: number; // Rating on a scale of 1 to 5

  @Attribute(DataTypes.INTEGER)
  declare parentId: CreationOptional<number>;

  @HasMany(() => Review, {
    foreignKey: "parentId",
    inverse: { as: "parenId" },
  })
  declare replies: NonAttribute<Review[]>;

  @Attribute(DataTypes.INTEGER)
  @Default(1) //limit tree sections: max is 3, once a review is on level 3 it can't have a reply
  declare level: CreationOptional<number>;

  @CreatedAt
  declare createdAt: CreationOptional<Date>;

  @UpdatedAt
  declare updatedAt: CreationOptional<Date>;
}
